# Functions for mx-pipeline scripts.
# This file is sourced by both mxp and mxp0 scripts.

# Join (like Python join):  join "separator" w1 ... wN {{{
join () {
    local s="$1"
    shift
    echo -n "$1"
    shift
    while (($# > 0)); do
        echo -n "$s$1"
        shift
    done
}
#}}}

# Just for better code formatting: {{{
#   'setv a B'  is equivalent to  'export -n a=B'
#   'setv -f a B C'  is equivalent to  'if [ -f B ]; then export -n a=B; else export -n a=C; fi'
# (any other valid bash test may be used in place of '-f').
#
# setvx also exports variable 'a'; setv always unexports variable.
setv () {
    if [ -z "$4" ]; then
        export -n $1="$2"
    else
        if [ "$1" "$3" ]; then export -n $2="$3"; else export -n $2="$4"; fi
    fi
}
setvx () {
    if [ -z "$4" ]; then
        export $1="$2"
    else
        if [ "$1" "$3" ]; then export $2="$3"; else export $2="$4"; fi
    fi
}
#}}}

# Expand string to required width. Argumets: prefix, fill-char, suffix, total width. {{{
# 'total-width' may be omitted, then value of $MXP_OUTPUT_WIDTH is used.
pad_string () {
    local out_width=${4:-$MXP_OUTPUT_WIDTH}
    local fill_width=$(( $out_width - (${#1} + ${#3}) ))
    local fill=""
    while ((fill_width > 0)); do
        fill=$2$fill
        (( fill_width-- ))
    done
    echo "$1$fill$3"
}
#}}}

# Current date/time formatted in desired manner {{{
curr_datetime () {
    date "+%Y-%m-%d %H:%M:%S"
}
#}}}


# Functions that are not used -- but may be used in the future. {{{

# Various tests with appropriate messaging {{{
# Usage:    check???  [-o]  [-v]  path  text-name
#   -o : absence is OK
#   -v : print any message, without -v -- only error message
checkdir () {
    local opt=0 vbt=0
    if [ "$1" == "-o" ]; then opt=1; shift; fi
    if [ "$1" == "-v" ]; then vbt=1; shift; fi
    if [ ! -e "$1" ]; then
        if (($opt)); then
            local  rc=0  msg1="[OK-]"  msg2=""
        else
            local  rc=1  msg1="[ERR]"  msg2="-- directory does not exist."
        fi
    elif [ ! -d "$1" ]; then
        local  rc=1  msg1="[ERR]"  msg2="-- exists but is not a directory."
    else
        local  rc=0  msg1="[OK+]"  msg2=""
    fi
    if (($vbt || $rc)); then
        echo "$msg1  $2  $1  $msg2"
    fi
    return $rc
}
checkfile () {
    local opt=0 vbt=0
    if [ "$1" == "-o" ]; then opt=1; shift; fi
    if [ "$1" == "-v" ]; then vbt=1; shift; fi
    if [ ! -e "$1" ]; then
        if (($opt)); then
            local  rc=0  msg1="[OK-]"  msg2=""
        else
            local  rc=1  msg1="[ERR]"  msg2="-- file does not exist."
        fi
    elif [ ! -f "$1" ]; then
        local  rc=1  msg1="[ERR]"  msg2="-- exists but is not a file."
    else
        local  rc=0  msg1="[OK+]"  msg2=""
    fi
    if (($vbt || $rc)); then
        echo "$msg1  $2  $1  $msg2"
    fi
    return $rc
}
#}}}

# Primarily for debugging: {{{
# Print global variables with names provided as arguments.
print_vars () { #{{{
    local varname maxwidth=0
    for varname; do
        maxwidth=$(( $maxwidth > ${#varname} ? $maxwidth : ${#varname} ))
    done
    printf "%*s-----\n" $maxwidth | tr " " "-"
    for varname; do
        printf "%*s  =  %s\n" $maxwidth $varname "${!varname}"
    done
    printf "%*s-----\n" $maxwidth | tr " " "-"
} #}}}
#}}}

#}}}
